@@ -103,7 +103,7 @@ char genCA[] =
 "private_key	= $dir/private/cakey.pem # The private key\n"
 "RANDFILE	= $dir/private/.rand	# private random number file\n"
 "\n"
-"x509_extensions	= usr_cert		# The extentions to add to the cert\n"
+"x509_extensions	= usr_cert		# The extensions to add to the cert\n"
 "\n"
 "name_opt 	= ca_default		# Subject Name options\n"
 "cert_opt 	= ca_default		# Certificate field options\n"
@@ -139,7 +139,7 @@ char genCA[] =
 "default_keyfile 	= privkey.pem\n"
 "distinguished_name	= req_distinguished_name\n"
 "attributes		= req_attributes\n"
-"x509_extensions	= v3_ca	# The extentions to add to the self signed cert\n"
+"x509_extensions	= v3_ca	# The extensions to add to the self signed cert\n"
 "\n"
 "string_mask = nombstr\n"
 "\n"
