@@ -54,6 +54,11 @@ so, delete this exception statement from your version.
 #include <linux/input.h>
 #include <linux/uinput.h>
 
+#ifndef input_event_sec
+#define input_event_sec time.tv_sec
+#define input_event_usec time.tv_usec
+#endif
+
 #if !defined(EV_SYN) || !defined(SYN_REPORT)
 #undef UINPUT_OK
 #endif
@@ -108,6 +113,8 @@ static int dragskip = 0;
 static int touch_always = 0;
 static int touch_pressure = 1;
 static int abs_x = 0, abs_y = 0;
+static int multitouch = 0;
+static int tracking_id = 1;
 
 static char *devs[] = {
 	"/dev/misc/uinput",
@@ -425,6 +432,8 @@ int initialize_uinput(void) {
 	ioctl(fd, UI_SET_EVBIT, EV_REL);
 	ioctl(fd, UI_SET_RELBIT, REL_X);
 	ioctl(fd, UI_SET_RELBIT, REL_Y);
+	ioctl(fd, UI_SET_RELBIT, REL_WHEEL);
+	ioctl(fd, UI_SET_RELBIT, REL_WHEEL_HI_RES);
 
 	ioctl(fd, UI_SET_EVBIT, EV_KEY);
 
@@ -698,6 +707,8 @@ void parse_uinput_str(char *in) {
 			}
 		} else if (strstr(p, "tslib_cal=") == p) {
 			tslib_cal = strdup(p+strlen("tslib_cal="));
+		} else if (strstr(p, "multitouch") == p) {
+			multitouch = 1;
 		} else {
 			rfbLog("invalid UINPUT option: %s\n", p);
 			clean_up_exit(1);
@@ -710,6 +721,7 @@ void parse_uinput_str(char *in) {
 static void ptr_move(int dx, int dy) {
 #ifdef UINPUT_OK
 	struct input_event ev;
+	struct timeval tval;
 	int d = direct_rel_fd < 0 ? fd : direct_rel_fd;
 
 	if (injectable && strchr(injectable, 'M') == NULL) {
@@ -720,7 +732,9 @@ static void ptr_move(int dx, int dy) {
 
 	if (db) fprintf(stderr, "ptr_move(%d, %d) fd=%d\n", dx, dy, d);
 
-	gettimeofday(&ev.time, NULL);
+	gettimeofday(&tval, NULL);
+	ev.input_event_sec = tval.tv_sec;
+	ev.input_event_usec = tval.tv_usec;
 	ev.type = EV_REL;
 	ev.code = REL_Y;
 	ev.value = dy;
@@ -755,6 +769,7 @@ static void apply_tslib(int *x, int *y) {
 static void ptr_abs(int x, int y, int p) {
 #ifdef UINPUT_OK
 	struct input_event ev;
+	struct timeval tval;
 	int x0, y0;
 	int d = direct_abs_fd < 0 ? fd : direct_abs_fd;
 
@@ -773,7 +788,9 @@ static void ptr_abs(int x, int y, int p) {
 
 	if (db) fprintf(stderr, "ptr_abs(%d, %d => %d %d, p=%d) fd=%d\n", x0, y0, x, y, p, d);
 
-	gettimeofday(&ev.time, NULL);
+	gettimeofday(&tval, NULL);
+	ev.input_event_sec = tval.tv_sec;
+	ev.input_event_usec = tval.tv_usec;
 	ev.type = EV_ABS;
 	ev.code = ABS_Y;
 	ev.value = y;
@@ -784,6 +801,18 @@ static void ptr_abs(int x, int y, int p) {
 	ev.value = x;
 	write(d, &ev, sizeof(ev));
 
+	if (multitouch) {
+		ev.type = EV_ABS;
+		ev.code = ABS_MT_POSITION_Y;
+		ev.value = y;
+		write(d, &ev, sizeof(ev));
+
+		ev.type = EV_ABS;
+		ev.code = ABS_MT_POSITION_X;
+		ev.value = x;
+		write(d, &ev, sizeof(ev));
+	}
+
 	if (p >= 0) {
 		ev.type = EV_ABS;
 		ev.code = ABS_PRESSURE;
@@ -796,6 +825,17 @@ static void ptr_abs(int x, int y, int p) {
 			ev.value = p ? 1 : 0;
 			write(d, &ev, sizeof(ev));
 		}
+
+		if (multitouch) {
+			ev.type = EV_ABS;
+			ev.code = ABS_MT_TRACKING_ID;
+			if (p==0){
+				ev.value = -1;
+			} else {
+				ev.value = tracking_id++;
+			}
+			write(d, &ev, sizeof(ev));
+		}
 	}
 
 	ev.type = EV_SYN;
@@ -950,6 +990,7 @@ if (0) {usleep(100*1000) ;}
 static void button_click(int down, int btn) {
 #ifdef UINPUT_OK
 	struct input_event ev;
+	struct timeval tval;
 	int d = direct_btn_fd < 0 ? fd : direct_btn_fd;
 
 	if (injectable && strchr(injectable, 'B') == NULL) {
@@ -959,7 +1000,9 @@ static void button_click(int down, int btn) {
 	if (db) fprintf(stderr, "button_click: btn %d %s fd=%d\n", btn, down ? "down" : "up", d);
 
 	memset(&ev, 0, sizeof(ev));
-	gettimeofday(&ev.time, NULL);
+	gettimeofday(&tval, NULL);
+	ev.input_event_sec = tval.tv_sec;
+	ev.input_event_usec = tval.tv_usec;
 	ev.type = EV_KEY;
 	ev.value = down;
 
@@ -973,9 +1016,19 @@ static void button_click(int down, int btn) {
 	} else if (btn == 3) {
 		ev.code = BTN_RIGHT;
 	} else if (btn == 4) {
-		ev.code = BTN_FORWARD;
+		ev.code = REL_WHEEL;
+		ev.type = EV_REL;
+		ev.value = 1;
+		write(d, &ev, sizeof(ev));
+		ev.code = REL_WHEEL_HI_RES;
+		ev.value = 120;
 	} else if (btn == 5) {
-		ev.code = BTN_BACK;
+		ev.code = REL_WHEEL;
+		ev.type = EV_REL;
+		ev.value = -1;
+		write(d, &ev, sizeof(ev));
+		ev.code = REL_WHEEL_HI_RES;
+		ev.value = -120;
 	} else {
 		return;
 	}
@@ -1230,6 +1283,7 @@ void uinput_pointer_command(int mask, int x, int y, rfbClientPtr client) {
 void uinput_key_command(int down, int keysym, rfbClientPtr client) {
 #ifdef UINPUT_OK
 	struct input_event ev;
+	struct timeval tval;
 	int scancode;
 	allowed_input_t input;
 	int d = direct_key_fd < 0 ? fd : direct_key_fd;
@@ -1253,7 +1307,9 @@ void uinput_key_command(int down, int keysym, rfbClientPtr client) {
 	if (db) fprintf(stderr, "uinput_key_command: %d -> %d %s fd=%d\n", keysym, scancode, down ? "down" : "up", d);
 
 	memset(&ev, 0, sizeof(ev));
-	gettimeofday(&ev.time, NULL);
+	gettimeofday(&tval, NULL);
+	ev.input_event_sec = tval.tv_sec;
+	ev.input_event_usec = tval.tv_usec;
 	ev.type = EV_KEY;
 	ev.code = (unsigned char) scancode;
 	ev.value = down;
@@ -1285,6 +1341,8 @@ static int lookup_code(int keysym) {
 
 	switch(keysym) {
 #ifdef UINPUT_OK
+	case XK_Super_L: return KEY_LEFTMETA;
+	case XK_Super_R: return KEY_RIGHTMETA;
 	case XK_Escape:	return KEY_ESC;
 	case XK_1:		return KEY_1;
 	case XK_2:		return KEY_2;
@@ -1484,6 +1542,8 @@ while (<>) {
 This only handles US kbd, we would need a kbd database in general...
 Ugh: parse dumpkeys(1) or -fookeys /usr/share/keymaps/i386/qwerty/dk.kmap.gz
 
+XK_Super_L	KEY_LEFTMETA
+XK_Super_R	KEY_RIGHTMETA
 XK_Escape	KEY_ESC
 XK_1		KEY_1
 XK_2		KEY_2
