@@ -69,6 +69,13 @@ so, delete this exception statement from your version.
 #include <rfb/rfb.h>
 #include <rfb/rfbregion.h>
 
+#if HAVE_LIBDRM
+/* drm access */
+#include <sys/ioctl.h>
+#include <xf86drm.h>
+#include <xf86drmMode.h>
+
+#endif
 
 /* we can now build under --without-x: */
 #if HAVE_X11
@@ -541,6 +548,8 @@ extern char *rot_fb;			/* used under -rotate */
 extern char *raw_fb;
 extern char *raw_fb_addr;
 extern int raw_fb_offset;
+extern int raw_fb_virt_x;
+extern int raw_fb_virt_y;
 extern int raw_fb_shm;
 extern int raw_fb_mmap;
 extern int raw_fb_seek;
